local addon = LibStub("AceAddon-3.0"):GetAddon("ErebusUI");
local module = addon:NewModule("ActionBars");
----------------------------------------------------------------------------------------------------
local default, plate = {
	bar1 = {alpha = 1, enable = 1},
	bar2 = {alpha = 1, enable = 1},
	bar3 = {alpha = 1, enable = 1},
	bar4 = {alpha = 1, enable = 1},
	bar5 = {alpha = 1, enable = 1},
	bar6 = {alpha = 1, enable = 1},
};

local UpdateSettings = function()
	DB.ActionBars = DB.ActionBars or {};
	for key,val in pairs(default) do
		if (not DB.ActionBars[key]) then DB.ActionBars[key] = val; end
		setmetatable(DB.ActionBars[key],{__index = default[key]});
	end
end;

local SetupProfile = function()
	UpdateSettings();
end;

local BartenderProfileCheck = function(Input,Report)
	local profiles, r = Bartender4.db:GetProfiles(), false
	for k,v in pairs(profiles) do if v == Input then r = true end end
	if Report then if r ~= true then addon:Print(Input.." Profile not found, generating one.") end end
	return r
end

-- Debug function to ease code testing while in-game
function p()
end

local SetupBartender = function()
	if (not select(4, GetAddOnInfo("Bartender4"))) then return; end
	local standard = "ErebusUI 3.1.4";
	local settings = { -- actual settings being inserted into our custom profile
			ActionBars = {
				actionbars = { -- following settings are bare minimum, so that anything not defined is retained between resets
					{enabled = true,	buttons = 12,	rows = 1,	padding = 12,	skin = {Zoom = true},	position = {point = "CENTER",	parent = "EUI_ActionBarPlate",	x=-243.95002,	y=90,	scale = 0.85,	growHorizontal="RIGHT"}}, -- 1
					{enabled = true,	buttons = 12,	rows = 1,	padding = 12,	skin = {Zoom = true},	position = {point = "CENTER",	parent = "EUI_ActionBarPlate",	x=-243.95002,	y=50,	scale = 0.85,	growHorizontal="RIGHT"}}, -- 2
					{enabled = true,	buttons = 12,	rows = 1,	padding = 12,	skin = {Zoom = true},	position = {point = "CENTER",	parent = "EUI_ActionBarPlate",	x=-243.95002,	y=10,	scale = 0.85,	growHorizontal="RIGHT"}}, -- 3
					{enabled = true,	buttons = 5,	rows = 1,	padding = 12,	skin = {Zoom = true},	position = {point = "CENTER",	parent = "EUI_ActionBarPlate",	x=-478,	y=90,	scale = 0.85,	growHorizontal="RIGHT"}}, -- 4
					{enabled = true,	buttons = 5,	rows = 1,	padding = 12,	skin = {Zoom = true},	position = {point = "CENTER",	parent = "EUI_ActionBarPlate",	x=-478,	y=50,	scale = 0.85,	growHorizontal="RIGHT"}}, -- 5
					{enabled = true,	buttons = 5,	rows = 1,	padding = 12,	skin = {Zoom = true},	position = {point = "CENTER",	parent = "EUI_ActionBarPlate",	x=-478,	y=10,	scale = 0.85,	growHorizontal="RIGHT"}}, -- 6
					{enabled = false,	buttons = 12,	rows = 1,	padding = 3,	skin = {Zoom = true},	position = {					parent = "EUI_ActionBarPlate",					scale = 0.85,	growHorizontal="RIGHT"}}, -- 7
					{enabled = false,	buttons = 12,	rows = 1,	padding = 3,	skin = {Zoom = true},	position = {					parent = "EUI_ActionBarPlate",					scale = 0.85,	growHorizontal="RIGHT"}}, -- 8
					{enabled = false,	buttons = 12,	rows = 1,	padding = 3,	skin = {Zoom = true},	position = {					parent = "EUI_ActionBarPlate",					scale = 0.85,	growHorizontal="RIGHT"}}, -- 9
					{enabled = false,	buttons = 12,	rows = 1,	padding = 3,	skin = {Zoom = true},	position = {					parent = "EUI_ActionBarPlate",					scale = 0.85,	growHorizontal="RIGHT"}} -- 10
				}
			},
			BagBar		= {	enabled = true, padding = 9, 		position = {point = "BOTTOM",		parent = "EUI_ActionBarPlate",	x=68.60002,			y=26,	scale = 0.70,	growHorizontal="LEFT"},		rows = 1, onebag = false, keyring = true},
			--
			MicroMenu	= {	enabled = true,	padding = -2,		position = {point = "CENTER",		parent = "EUI_ActionBarPlate",	x=-499.5002,	y=135,	scale = 0.75,	growHorizontal="RIGHT"}},
			--
			PetBar		= {	enabled = true, padding = 2, 		position = {point = "CENTER",		parent = "EUI_ActionBarPlate",	x=-244,	y=131,	scale = 0.70,	growHorizontal="RIGHT"},	rows = 1, skin = {Zoom = true}},
			StanceBar	= {	enabled = true,	padding = 2, 		position = {point = "CENTER",		parent = "EUI_ActionBarPlate",	x=252,	y=131,	scale = 0.70,	growHorizontal="LEFT"},		rows = 1, skin = {Zoom = true}},
			MultiCast	= {	enabled = true,						position = {point = "TOPRIGHT",		parent = "EUI_ActionBarPlate",	x=-777,	y=-4,	scale = 0.75}},
			Vehicle		= {	enabled = false,	padding = 3,		position = {point = "CENTER",		parent = "EUI_ActionBarPlate",	x=-15,	y=213,	scale = 0.85}},
			ExtraActionBar = {	enabled = true,					position = {point = "CENTER",		parent = "EUI_ActionBarPlate",	x=-32,	y=240}},
			BlizzardArt	= {	enabled = false,	},
		};
	
	local lib = LibStub("LibWindow-1.1",true);
	if not lib then return; end
	function lib.RegisterConfig(frame, storage, names)
		if not lib.windowData[frame] then
			lib.windowData[frame] = {}
		end
		lib.windowData[frame].names = names
		lib.windowData[frame].storage = storage
		local parent = frame:GetParent();
		if (storage.parent) then
			frame:SetParent(storage.parent);
			if storage.parent == "EUI_ActionBarPlate" then
				frame:SetFrameStrata("LOW");
			end
		elseif (parent and parent:GetName() == "EUI_ActionBarPlate") then
			frame:SetParent(UIParent);
		end
	end
	SetupProfile = function() -- apply default settings into a custom BT4 profile
		UpdateSettings();
		-- New check for updating old profiles
		if ( not DB.ActionBars.ErebusUI_Version) then
			DB.ActionBars.ErebusUI_Version = GetAddOnMetadata("ErebusUI", "Version");
			-- Update Blizzard Art Bar settings on the standard profile
			if BartenderProfileCheck(standard,false) then
				for k,v in LibStub("AceAddon-3.0"):IterateModulesOfAddon(Bartender4) do -- for each module (BagBar, ActionBars, etc..)
					if v.db and v.db.profile then
						-- Fixup Blizzard Art Bar
						v.db.profile["BlizzardArt"] = module:MergeData(v.db.profile["BlizzardArt"],settings["BlizzardArt"])
						-- Fixup Vehicle Bar
						v.db.profile["Vehicle"] = module:MergeData(v.db.profile["Vehicle"],settings["Vehicle"])
					end
				end
			end
		end
		--print("Using this profile: "..Bartender4.db:GetCurrentProfile())
		-- Checking for the standard Profile
		if (not BartenderProfileCheck(standard,true)) then DB.ActionBars.Bartender4 = false end
		-- Fixup setting profile to standard if standard profile exist
		if DB.ActionBars.Bartender4 then if profile == standard then Bartender4.db:SetProfile(standard) end return; end
		Bartender4.db:SetProfile(standard);
		for k,v in LibStub("AceAddon-3.0"):IterateModulesOfAddon(Bartender4) do -- for each module (BagBar, ActionBars, etc..)
			if settings[k] and v.db.profile then
				v.db.profile = module:MergeData(v.db.profile,settings[k])
			end
		end
		Bartender4:UpdateModuleConfigs(); -- run ApplyConfig for all modules, so that the new settings are applied
		if BartenderProfileCheck(standard,false) then addon:Print(standard.." Profile generated in Bartender.") end
		DB.ActionBars.Bartender4 = true;
	end
	-- Can't use UpdateInterval, due to the way this need to be working -- could this behavior be changed? - Maybe a securehoocfunc on the unlocking
	plate:HookScript("OnUpdate",function(self,...)
		if (InCombatLockdown()) then return; end
		if (Bartender4.db:GetCurrentProfile() == standard) or (Bartender4.db:GetCurrentProfile() == newtest) then
			if Bartender4.Locked then return; end
			addon:Print("The ability to unlock your bars is disabled when using the ErebusUI Default profile in Bartender4. Please change profiles to enable this functionality.");
			Bartender4:Lock();
		end
	end);
end;

function module:MergeData(target,source)
	if type(target) ~= "table" then target = {} end
	for k,v in pairs(source) do
		if type(v) == "table" then
			target[k] = self:MergeData(target[k], v);
		else
			target[k] = v;
		end
	end
	return target;
end

function module:OnInitialize()
	do -- create bar plate and masks
		plate = CreateFrame("Frame","EUI_ActionBarPlate",ErebusUI,"EUI_ActionBarsTemplate");
		plate:SetFrameStrata("BACKGROUND"); plate:SetFrameLevel(1);
		plate:SetPoint("BOTTOM");
	end
	SetupBartender();
end

function module:OnEnable()
	do -- create base module frames
		-- Fix CPU leak, use UpdateInterval
		plate.UpdateInterval = 0.5
		plate.TimeSinceLastUpdate = 0
		plate:HookScript("OnUpdate",function(self,...) -- backdrop visibility changes (alpha, animation, hide/show)
			local elapsed = select(1,...)
			self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed; 
			if (self.TimeSinceLastUpdate > self.UpdateInterval) then
				-- Debug
--				print(self.TimeSinceLastUpdate)
				if (DB.ActionBars.bar1) then
					for b = 1, 6 do -- for each backdrop
						if DB.ActionBars["bar"..b].enable then -- backdrop enabled
							_G["EUI_Bar"..b]:SetAlpha(DB.ActionBars["bar"..b].alpha/100 or 1); -- apply alpha
						else -- backdrop disabled
							_G["EUI_Bar"..b]:SetAlpha(0);
						end
					end

				end
				self.TimeSinceLastUpdate = 0
			end
		end);
	end
	do -- modify strata / levels of backdrops
		for i = 1,6 do
			_G["EUI_Bar"..i]:SetFrameStrata("BACKGROUND");
			_G["EUI_Bar"..i]:SetFrameLevel(3);
		end
	end
	SetupProfile();
	-- Do what Bartender isn't - Make the Bag buttons the same size
--	do -- modify CharacterBag(0-3) Scale
--		for i = 1,4 do
--			_G["CharacterBag"..(i-1).."Slot"]:SetScale(1.25);
--		end
--	end
end